searchData={"items":[{"type":"module","title":"ExAws.Dynamo","doc":"Operations on the AWS DynamoDB service.\n\nNOTE: When Mix.env in [:test, :dev], Dynamo clients will run by default against\nDynamoDB local.","ref":"ExAws.Dynamo.html"},{"type":"module","title":"Basic usage - ExAws.Dynamo","doc":"```elixir\ndefmodule User do\n  @derive [ExAws.Dynamo.Encodable]\n  defstruct [:email, :name, :age, :admin]\nend\n\nalias ExAws.Dynamo\n\n# Create a provisioned users table with a primary key of email [String]\n# and 1 unit of read and write capacity\nDynamo.create_table(\"Users\", \"email\", %{email: :string}, 1, 1)\n|> ExAws.request!\n\nuser = %User{email: \"bubba@foo.com\", name: \"Bubba\", age: 23, admin: false}\n# Save the user\nDynamo.put_item(\"Users\", user) |> ExAws.request!\n\n# Retrieve the user by email and decode it as a User struct.\nresult = Dynamo.get_item(\"Users\", %{email: user.email})\n|> ExAws.request!\n|> Dynamo.decode_item(as: User)\n\nassert user == result\n```","ref":"ExAws.Dynamo.html#module-basic-usage"},{"type":"module","title":"General notes - ExAws.Dynamo","doc":"All options are handled as underscored atoms instead of camelcased binaries as specified\nin the Dynamo API, e.g. `IndexName` would be `:index_name`. Anywhere in the API that requires\nDynamo type annotation (`{\"S\":\"mystring\"}`) is handled for you automatically. For example,\n\n```elixir\nExAws.Dynamo.scan(\"Users\", expression_attribute_values: [api_key: \"foo\"])\n```\ntransforms into a query of\n```elixir\n%{\"ExpressionAttributeValues\" => %{api_key: %{\"S\" => \"foo\"}}, \"TableName\" => \"Users\"}\n```\n\nConsult the function documentation to see precisely which options are handled this way.\n\nIf you wish to avoid this kind of automatic behaviour, you are free to specify the types yourself.\nFor example,\n```elixir\nExAws.Dynamo.scan(\"Users\", expression_attribute_values: [api_key: %{\"B\" => \"Treated as binary\"}])\n```\nbecomes\n```elixir\n%{\"ExpressionAttributeValues\" => %{api_key: %{\"B\" => \"Treated as binary\"}}, \"TableName\" => \"Users\"}\n```\nAlternatively, if what's being encoded is a struct, you're always free to implement ExAws.Dynamo.Encodable for that struct.\n\nhttps://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_Operations.html","ref":"ExAws.Dynamo.html#module-general-notes"},{"type":"function","title":"ExAws.Dynamo.batch_get_item/2","doc":"Batch-get up to 100 items (16 MB total max)\n\nMap of table names to request parameter maps.\nhttps://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_BatchGetItem.html\n\nParameters with keys that are automatically annotated with dynamo types are:\n`[:keys]`\n\n```elixir\nDynamo.batch_get_item(%{\n  \"Users\" => [\n    consistent_read: true,\n    keys: [\n      [api_key: \"key1\"],\n      [api_key: \"api_key2\"]\n    ],\n    expression_attribute_names: %{\"#api_key\" => \"api_key\"},\n    projection_expression: \"#api_key\"\n  ],\n  \"Subscriptions\" => %{\n    keys: [\n      %{id: \"id1\"}\n    ]\n  }\n})\n```\nAs you see, you're largely free to use either keyword args or maps in the body. A map\nis required for the argument itself because the table names are most often binaries, and I refuse\nto inflict proplists on anyone.","ref":"ExAws.Dynamo.html#batch_get_item/2"},{"type":"function","title":"ExAws.Dynamo.batch_write_item/2","doc":"Put or delete up to 25 items (16 MB total max)\n\nMap of table names to request parameter maps.\nhttps://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_BatchWriteItem.html\n\nParameters with keys that are automatically annotated with Dynamo types are:\n`[:keys]`","ref":"ExAws.Dynamo.html#batch_write_item/2"},{"type":"function","title":"ExAws.Dynamo.create_table/4","doc":"Create table with arbitrary options. This provides a superset of the functionality of other\n`create_table` variants, allowing the specification of any options as per the `t:create_table_opts/0`\ntype. If no options are specified, defaults will be used as follows:\n\n* `billing_mode`: `:provisioned`\n* `read_capacity`: `10`\n* `write_capacity`: `10`\n* `stream_enabled`: `false`","ref":"ExAws.Dynamo.html#create_table/4"},{"type":"function","title":"ExAws.Dynamo.create_table/6","doc":"Create table\n\n`key_schema` can be a simple binary or atom indicating a simple hash key.\n\n`billing_mode` may be either `:provisioned` (default) or `:pay_per_request`.\nIf you are creating a `:pay-per-request` table, you will still need to provide values for read and write capacities,\nalthough they will be ignored - you may consider providing `nil` in those cases.","ref":"ExAws.Dynamo.html#create_table/6"},{"type":"function","title":"ExAws.Dynamo.create_table/8","doc":"Create table with secondary indices\n\nEach index should follow the format outlined here: https://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_CreateTable.html\n\nFor convenience, the keys in each index map are allowed to be atoms. e.g:\n`\"KeySchema\"` in the aws docs can be `key_schema:`\n\nNote that both the `global_indexes` and `local_indexes` arguments expect a list of such indices.\n\n`billing_mode` may be either `:provisioned` (default) or `:pay_per_request`.\nIf you are creating a `:pay-per-request` table, you will still need to provide values for read and write capacities,\nalthough they will be ignored - you may consider providing `nil` in those cases.\n\nExamples\n```\nsecondary_index = [%{\n  index_name: \"my-global-index\",\n  key_schema: [%{\n    attribute_name: \"email\",\n    key_type: \"HASH\",\n  }],\n  provisioned_throughput: %{\n    read_capacity_units: 1,\n    write_capacity_units: 1,\n  },\n  projection: %{\n    projection_type: \"KEYS_ONLY\",\n  }\n}]\ncreate_table(\"TestUsers\", [id: :hash], %{id: :string, email: :string}, 1, 1, secondary_index, [])\n```","ref":"ExAws.Dynamo.html#create_table/8"},{"type":"function","title":"ExAws.Dynamo.decode_item/2","doc":"Decode an item returned from Dynamo. This will handle items wrapped in the ordinary\n`get_item` response map of `%{\"Item\" => item}`.","ref":"ExAws.Dynamo.html#decode_item/2"},{"type":"function","title":"Example - ExAws.Dynamo.decode_item/2","doc":"```elixir\nDynamo.get_item(\"users\", %{id: \"asdf\"})\n|> ExAws.request!\n|> Dynamo.decode_item(as: User)\n```","ref":"ExAws.Dynamo.html#decode_item/2-example"},{"type":"function","title":"ExAws.Dynamo.delete_item/3","doc":"Delete item in table","ref":"ExAws.Dynamo.html#delete_item/3"},{"type":"function","title":"ExAws.Dynamo.delete_table/1","doc":"Delete Table","ref":"ExAws.Dynamo.html#delete_table/1"},{"type":"function","title":"ExAws.Dynamo.describe_table/1","doc":"Describe table","ref":"ExAws.Dynamo.html#describe_table/1"},{"type":"function","title":"ExAws.Dynamo.describe_time_to_live/1","doc":"Describe time to live","ref":"ExAws.Dynamo.html#describe_time_to_live/1"},{"type":"function","title":"ExAws.Dynamo.get_item/3","doc":"Get item from table","ref":"ExAws.Dynamo.html#get_item/3"},{"type":"function","title":"ExAws.Dynamo.list_tables/0","doc":"List tables","ref":"ExAws.Dynamo.html#list_tables/0"},{"type":"function","title":"ExAws.Dynamo.put_item/3","doc":"Put item in table","ref":"ExAws.Dynamo.html#put_item/3"},{"type":"function","title":"ExAws.Dynamo.query/2","doc":"Query Table\n\nPlease read https://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_Query.html\n\n```\nDynamo.query(\"Users\",\n  limit: 1,\n  expression_attribute_values: [desired_api_key: \"adminkey\"],\n  key_condition_expression: \"api_key = :desired_api_key\")\n```\n\nParameters with keys that are automatically annotated with dynamo types are:\n`[:exclusive_start_key, :expression_attribute_names]`","ref":"ExAws.Dynamo.html#query/2"},{"type":"function","title":"ExAws.Dynamo.scan/2","doc":"Scan table\n\nPlease read https://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_Scan.html\n\n```\nDynamo.scan(\"Users\"\n  limit: 1,\n  expression_attribute_values: [desired_api_key: \"adminkey\"],\n  expression_attribute_names: %{\"#asdf\" => \"api_key\"},\n  filter_expression: \"#asdf = :desired_api_key\")\n```\n\nGenerally speaking, you won't need to use `:expression_attribute_names`. It exists\nto alias a column name if one of the columns you want to search against is a reserved Dynamo word,\nlike `Percentile`. In this case, it's totally unnecessary as `api_key` is not a reserved word.\n\nParameters with keys that are automatically annotated with Dynamo types are:\n`[:exclusive_start_key, :expression_attribute_names]`","ref":"ExAws.Dynamo.html#scan/2"},{"type":"function","title":"ExAws.Dynamo.transact_get_items/2","doc":"A synchronous operation that retrieves multiple items from one or more tables (but not from indexes) in a single account and region","ref":"ExAws.Dynamo.html#transact_get_items/2"},{"type":"function","title":"ExAws.Dynamo.transact_write_items/2","doc":"A synchronous write operation that groups up to 25 action requests","ref":"ExAws.Dynamo.html#transact_write_items/2"},{"type":"function","title":"ExAws.Dynamo.update_item/3","doc":"Update item in table\n\nFor update_args format see\nhttps://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_UpdateItem.html","ref":"ExAws.Dynamo.html#update_item/3"},{"type":"function","title":"ExAws.Dynamo.update_table/2","doc":"Update Table","ref":"ExAws.Dynamo.html#update_table/2"},{"type":"function","title":"ExAws.Dynamo.update_time_to_live/3","doc":"Update time to live","ref":"ExAws.Dynamo.html#update_time_to_live/3"},{"type":"type","title":"ExAws.Dynamo.batch_get_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:batch_get_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.batch_write_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:batch_write_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.create_table_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:create_table_opts/0"},{"type":"type","title":"ExAws.Dynamo.delete_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:delete_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.dynamo_billing_types/0","doc":"","ref":"ExAws.Dynamo.html#t:dynamo_billing_types/0"},{"type":"type","title":"ExAws.Dynamo.dynamo_type_names/0","doc":"","ref":"ExAws.Dynamo.html#t:dynamo_type_names/0"},{"type":"type","title":"ExAws.Dynamo.exclusive_start_key_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:exclusive_start_key_vals/0"},{"type":"type","title":"ExAws.Dynamo.expression_attribute_names_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:expression_attribute_names_vals/0"},{"type":"type","title":"ExAws.Dynamo.expression_attribute_values_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:expression_attribute_values_vals/0"},{"type":"type","title":"ExAws.Dynamo.get_item/0","doc":"","ref":"ExAws.Dynamo.html#t:get_item/0"},{"type":"type","title":"ExAws.Dynamo.get_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:get_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.key_definitions/0","doc":"","ref":"ExAws.Dynamo.html#t:key_definitions/0"},{"type":"type","title":"ExAws.Dynamo.key_schema/0","doc":"","ref":"ExAws.Dynamo.html#t:key_schema/0"},{"type":"type","title":"ExAws.Dynamo.primary_key/0","doc":"","ref":"ExAws.Dynamo.html#t:primary_key/0"},{"type":"type","title":"ExAws.Dynamo.put_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:put_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.query_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:query_opts/0"},{"type":"type","title":"ExAws.Dynamo.return_consumed_capacity_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:return_consumed_capacity_vals/0"},{"type":"type","title":"ExAws.Dynamo.return_item_collection_metrics_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:return_item_collection_metrics_vals/0"},{"type":"type","title":"ExAws.Dynamo.return_values_on_condition_check_failure_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:return_values_on_condition_check_failure_vals/0"},{"type":"type","title":"ExAws.Dynamo.return_values_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:return_values_vals/0"},{"type":"type","title":"ExAws.Dynamo.scan_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:scan_opts/0"},{"type":"type","title":"ExAws.Dynamo.select_vals/0","doc":"","ref":"ExAws.Dynamo.html#t:select_vals/0"},{"type":"type","title":"ExAws.Dynamo.stream_view_type/0","doc":"","ref":"ExAws.Dynamo.html#t:stream_view_type/0"},{"type":"type","title":"ExAws.Dynamo.table_name/0","doc":"","ref":"ExAws.Dynamo.html#t:table_name/0"},{"type":"type","title":"ExAws.Dynamo.transact_get_item/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_get_item/0"},{"type":"type","title":"ExAws.Dynamo.transact_get_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_get_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.transact_get_items_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_get_items_opts/0"},{"type":"type","title":"ExAws.Dynamo.transact_standard_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_standard_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.transact_update_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_update_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.transact_write_item/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_write_item/0"},{"type":"type","title":"ExAws.Dynamo.transact_write_items_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:transact_write_items_opts/0"},{"type":"type","title":"ExAws.Dynamo.update_item_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:update_item_opts/0"},{"type":"type","title":"ExAws.Dynamo.update_table_opts/0","doc":"","ref":"ExAws.Dynamo.html#t:update_table_opts/0"},{"type":"type","title":"ExAws.Dynamo.write_item/0","doc":"","ref":"ExAws.Dynamo.html#t:write_item/0"},{"type":"protocol","title":"ExAws.Dynamo.Decodable","doc":"Allows custom decoding logic for your struct.","ref":"ExAws.Dynamo.Decodable.html"},{"type":"function","title":"ExAws.Dynamo.Decodable.decode/1","doc":"","ref":"ExAws.Dynamo.Decodable.html#decode/1"},{"type":"type","title":"ExAws.Dynamo.Decodable.t/0","doc":"All the types that implement this protocol.","ref":"ExAws.Dynamo.Decodable.html#t:t/0"},{"type":"module","title":"ExAws.Dynamo.Decoder","doc":"Decodes a Dynamo response into a struct.\n\nIf Dynamo.Decodable is implemented for the struct, it will be called\nafter the completion of the coercion.\n\nThis is important for handling nested maps if you wanted the nested maps\nto have atom keys.","ref":"ExAws.Dynamo.Decoder.html"},{"type":"function","title":"ExAws.Dynamo.Decoder.binary_map_to_struct/2","doc":"Converts a map with binary keys to the specified struct","ref":"ExAws.Dynamo.Decoder.html#binary_map_to_struct/2"},{"type":"function","title":"ExAws.Dynamo.Decoder.binary_to_number/1","doc":"Attempts to convert a number to a float, and then an integer","ref":"ExAws.Dynamo.Decoder.html#binary_to_number/1"},{"type":"function","title":"ExAws.Dynamo.Decoder.decode/1","doc":"Convert Dynamo format to Elixir\n\nFunctions which convert the Dynamo-style values into normal Elixir values.\nUse these if you just want the Dynamo result to look more like Elixir without\ncoercing it into a particular struct.","ref":"ExAws.Dynamo.Decoder.html#decode/1"},{"type":"function","title":"ExAws.Dynamo.Decoder.decode/2","doc":"","ref":"ExAws.Dynamo.Decoder.html#decode/2"},{"type":"protocol","title":"ExAws.Dynamo.Encodable","doc":"","ref":"ExAws.Dynamo.Encodable.html"},{"type":"function","title":"ExAws.Dynamo.Encodable.encode/2","doc":"Converts an Elixir value into a map tagging the value with its Dynamo type","ref":"ExAws.Dynamo.Encodable.html#encode/2"},{"type":"type","title":"ExAws.Dynamo.Encodable.t/0","doc":"","ref":"ExAws.Dynamo.Encodable.html#t:t/0"},{"type":"module","title":"ExAws.Dynamo.Encoder","doc":"Takes an Elixir value and converts it into a Dynamo-style map.\n\n```elixir\nMapSet.new [1,2,3] |> Elixir.ExAws.Dynamo.Encoder.encode\n#=> %{\"NS\" => [\"1\", \"2\", \"3\"]}\n\nMapSet.new [\"A\",\"B\",\"C\"] |> Elixir.ExAws.Dynamo.Encoder.encode\n#=> %{\"SS\" => [\"A\", \"B\", \"C\"]}\n\n\"bubba\" |> ExAws.Dynamo.Encoder.encode\n#=> %{\"S\" => \"bubba\"}\n```\n\nThis is handled via the ExAws.Dynamo.Encodable protocol.","ref":"ExAws.Dynamo.Encoder.html"},{"type":"function","title":"ExAws.Dynamo.Encoder.atom_to_dynamo_type/1","doc":"","ref":"ExAws.Dynamo.Encoder.html#atom_to_dynamo_type/1"},{"type":"function","title":"ExAws.Dynamo.Encoder.encode/1","doc":"","ref":"ExAws.Dynamo.Encoder.html#encode/1"},{"type":"function","title":"ExAws.Dynamo.Encoder.encode/2","doc":"","ref":"ExAws.Dynamo.Encoder.html#encode/2"},{"type":"function","title":"ExAws.Dynamo.Encoder.encode!/2","doc":"","ref":"ExAws.Dynamo.Encoder.html#encode!/2"},{"type":"function","title":"ExAws.Dynamo.Encoder.encode_root/2","doc":"","ref":"ExAws.Dynamo.Encoder.html#encode_root/2"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v4.2.2 - 2024-09-10 - Changelog","doc":"- Support condition failure return values\n- Fixes for warnings in Elixir 1.17","ref":"changelog.html#v4-2-2-2024-09-10"},{"type":"extras","title":"v4.2.1 - 2023-02-12 - Changelog","doc":"- Fixes spec","ref":"changelog.html#v4-2-1-2023-02-12"},{"type":"extras","title":"v4.2.0 - 2022-09-12 - Changelog","doc":"- Add info on transaction failures\n- Increase minimum ex_aws version to 2.4.0\n- Fix encoding of boolean attributes","ref":"changelog.html#v4-2-0-2022-09-12"},{"type":"extras","title":"v4.1.0 - 2022-03-23 - Changelog","doc":"- Add `:stream_enabled` and `:stream_view_type` options to table creation and updates\n- Increase minimum Elixir version to 1.10","ref":"changelog.html#v4-1-0-2022-03-23"},{"type":"extras","title":"v4.0.2 - 2021-12-24 - Changelog","doc":"- Add :consistent_read option in scan_opts spec\n- Add `code_quality` alias\n- Adds dialyxir to dev dependencies\n- Various credo and dialyzer fixes\n- Fixes typos in documentation and README","ref":"changelog.html#v4-0-2-2021-12-24"},{"type":"extras","title":"v4.0.1 - 2021-04-26 - Changelog","doc":"- Update dependencies\n- Update documentation","ref":"changelog.html#v4-0-1-2021-04-26"},{"type":"extras","title":"v4.0.0 - 2020-07-06 - Changelog","doc":"- Add support for empty string/binary attributes\n- Empty string is saved as a space when using `put_item`","ref":"changelog.html#v4-0-0-2020-07-06"},{"type":"extras","title":"v3.0.3 - 2020-07-03 - Changelog","doc":"- Update dependencies","ref":"changelog.html#v3-0-3-2020-07-03"},{"type":"extras","title":"v3.0.2 - 2020-05-23 - Changelog","doc":"- Add empty prod.exs config file","ref":"changelog.html#v3-0-2-2020-05-23"},{"type":"extras","title":"v3.0.1 - 2020-05-04 - Changelog","doc":"- Upcase `return_consumed_capacity` values on batch calls\n- Documentation updates and fixes","ref":"changelog.html#v3-0-1-2020-05-04"},{"type":"extras","title":"v3.0.0 - 2020-02-22 - Changelog","doc":"- Set minimum version of `ex_aws` to 2.1.2\n- Remove reference to `:decode_sets_config` option","ref":"changelog.html#v3-0-0-2020-02-22"},{"type":"extras","title":"v2.3.4 - 2019-10-31 - Changelog","doc":"- Additional documentation changes\n- Set `config/test.exs` as a 'bare-bones' config file\n- Import `ddb_local_test.exs` only if it has been provided.","ref":"changelog.html#v2-3-4-2019-10-31"},{"type":"extras","title":"v2.3.3 - 2019-10-24 - Changelog","doc":"- Allow `expression_attribute_names` to be specified in `batch_get_item`\n- Add badges to documentation","ref":"changelog.html#v2-3-3-2019-10-24"},{"type":"extras","title":"v2.3.2 - 2019-10-20 - Changelog","doc":"- Set `:debug_requests` to false in `test.exs.example`\n- Run integration tests only local DDB instance found","ref":"changelog.html#v2-3-2-2019-10-20"},{"type":"extras","title":"v2.3.1 - 2019-10-07 - Changelog","doc":"- Fix documentation","ref":"changelog.html#v2-3-1-2019-10-07"},{"type":"extras","title":"v2.3.0 - 2019-09-27 - Changelog","doc":"- Add DynamoDb transaction API calls\n- Code formatting","ref":"changelog.html#v2-3-0-2019-09-27"},{"type":"extras","title":"v2.2.2 - 2019-09-25 - Changelog","doc":"- Add instructions on running integration tests against local DDB","ref":"changelog.html#v2-2-2-2019-09-25"},{"type":"extras","title":"v2.2.1 - 2019-08-27 - Changelog","doc":"- Properly handle Binary fields","ref":"changelog.html#v2-2-1-2019-08-27"},{"type":"extras","title":"v2.2.0 - 2019-08-12 - Changelog","doc":"- Fix type `expression_attribute_values_vals`\n- Decode sets as MapSet","ref":"changelog.html#v2-2-0-2019-08-12"},{"type":"extras","title":"v2.1.0 - 2019-07-19 - Changelog","doc":"- Add support for billing mode\n- Add ability to specify billing mode and TTL during table create and update\n- Add support for list of items while decoding\n- Add `Dynamo.update_time_to_live` and `Dynamo.describe_time_to_live`","ref":"changelog.html#v2-1-0-2019-07-19"},{"type":"extras","title":"v2.0.0 - 2017-11-10 - Changelog","doc":"- Major Project Split. Please see the main [ExAws](https://github.com/ex-aws/ex_aws) repository for [previous changelogs](https://github.com/ex-aws/ex_aws/blob/master/CHANGELOG.md).","ref":"changelog.html#v2-0-0-2017-11-10"},{"type":"extras","title":"ExAws.Dynamo","doc":"# ExAws.Dynamo\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_aws_dynamo.svg)](https://hex.pm/packages/ex_aws_dynamo)\n[![Build Docs](https://img.shields.io/badge/hexdocs-release-blue.svg)](https://hexdocs.pm/ex_aws_dynamo/ExAws.Dynamo.html)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ex_aws_dynamo/)\n[![Total Download](https://img.shields.io/hexpm/dt/ex_aws_dynamo.svg)](https://hex.pm/packages/ex_aws_dynamo)\n[![License](https://img.shields.io/hexpm/l/ex_aws_dynamo.svg)](https://github.com/ex-aws/ex_aws_dynamo/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/ex-aws/ex_aws_dynamo.svg)](https://github.com/ex-aws/ex_aws_dynamo/commits/master)\n\nService module for https://github.com/ex-aws/ex_aws","ref":"readme.html"},{"type":"extras","title":"Installation - ExAws.Dynamo","doc":"The package can be installed by adding `:ex_aws_dynamo` to your list of dependencies in `mix.exs` along with your preferred JSON codec and HTTP client:\n\n```elixir\ndef deps do\n  [\n    {:ex_aws_dynamo, \"~> 4.0\"},\n    {:jason, \"~> 1.0\"},\n    {:hackney, \"~> 1.9\"}\n  ]\nend\n```\n\n**ExAws** uses [Jason](https://github.com/michalmuskala/jason) as its default JSON codec - if you intend to use a different codec, see [ex_aws](https://github.com/ex-aws/ex_aws) for more information about setting a custom `:json_codec`.\n\nDocumentation for **ExAwsDynamo** can be found at [https://hexdocs.pm/ex_aws_dynamo](https://hexdocs.pm/ex_aws_dynamo).","ref":"readme.html#installation"},{"type":"extras","title":"Requirements - ExAws.Dynamo","doc":"","ref":"readme.html#requirements"},{"type":"extras","title":"DynamoDB Local - ExAws.Dynamo","doc":"If you are running this module against a local development instance of DynamoDB, you'll want to make sure that you have installed the latest version, `1.17.2` (released 2021-12-16). You can find links to download the latest version [here](https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DynamoDBLocal.DownloadingAndRunning.html).","ref":"readme.html#dynamodb-local"},{"type":"extras","title":"Configuration - ExAws.Dynamo","doc":"### **ExAws**\n\nIn order to use **ExAws.Dynamo** against a production instance of DynamoDB, your app will need to provide the following configuration for `:ex_aws`:\n\nExample configuration:\n\n```elixir\nconfig :ex_aws,\n  access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, :instance_role],\n  secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, :instance_role]\n```\n\n**:access_key_id** :: string\n\nThe ID portion of a set of AWS credentials.\n\n**:secret_access_key** :: string\n\nThe secret portion of a set of AWS credentials.\n\nPlease see the [ExAws README](https://github.com/ex-aws/ex_aws) for additional reference.","ref":"readme.html#configuration"},{"type":"extras","title":"Testing - ExAws.Dynamo","doc":"","ref":"readme.html#testing"},{"type":"extras","title":"Integration tests (optional) - ExAws.Dynamo","doc":"The tests in `test/lib/dynamo/integration_test.exs` will attempt to run against a local instance of DynamoDB - in order to run these tests, you will need both a running local instance of DynamoDB as well as a `config/ddb_local_test.exs` file (.gitignored) formatted like so:\n\n`config/ddb_local_test.exs`\n\n```elixir\nimport Config\n\nconfig :ex_aws,\n  debug_requests: false, # set to true to monitor the DDB requests\n  access_key_id: \"abcd\",\n  secret_access_key: \"1234\",\n  region: \"us-east-1\"\n\nconfig :ex_aws, :dynamodb,\n  scheme: \"http://\",\n  host: \"localhost\",\n  port: SET_YOUR_PORT,\n  region: \"us-east-1\"\n```\n\n(or see `config/ddb_local_test.exs.example`)\n\nBefore running tests, be aware that `test/lib/dynamo/integration_test.exs` will create and delete tables with the names `\"TestUsers\"`, `\"Test.User\"`, `\"TestSeveralUsers\"`, `\"TestFoo\"`, `\"test_books\"`, `\"TestUsersWithRange\"`, `\"TestTransactions\"`, `\"TestTransactions2\"` - test operations may affect your current tables if they share any of those names. You may consider maintaining a separate instance of DynamoDB local (or at least a separate instance of its `shared-local-instance.db` file) for testing.\n\nIf DynamoDB is not running locally or the config file has not been provided, the integration tests will be skipped.","ref":"readme.html#integration-tests-optional"},{"type":"extras","title":"Copyright and License - ExAws.Dynamo","doc":"The MIT License (MIT)\n\nCopyright (c) 2014 CargoSense, Inc.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Upgrading","doc":"# Upgrading","ref":"upgrade.html"},{"type":"extras","title":"3.X.X to 4.X.X - Upgrading","doc":"","ref":"upgrade.html#3-x-x-to-4-x-x"},{"type":"extras","title":"Empty string/binary values allowed - Upgrading","doc":"In May, 2020, DynamoDB was updated to allow storage of empty strings and binaries in non-key fields - [documentation](https://aws.amazon.com/about-aws/whats-new/2020/05/amazon-dynamodb-now-supports-empty-values-for-non-key-string-and-binary-attributes-in-dynamodb-tables/). In earlier versions of this app, fields with empty string attributes were stripped off prior to writing to the DB by `ExAws.Dynamo.Encodable.do_encode/1`; however, in light of these developments, the default behavior of this app will now be to write empty string values.\n\nPlease check your application thoroughly and make sure you are prepared for dealing with the possibility of having empty strings in fields that may previously have been expected to return `nil` (since, in the past, they wouldn't have been written at all). To be clear, existing data is not going to be changed - but if, prior to upgrading to version 4, you wrote a record with the values `%{id: \"1\", optional_string: \"\"}`, that record will still be returned as `%{id: 1, optional_string: nil}`; if you write a record after upgrading, say `%{id: \"2\", optional_string: \"\"}`, that record will be returned exactly as written, `%{id: \"2\", optional_string: \"\"}`.","ref":"upgrade.html#empty-string-binary-values-allowed"},{"type":"extras","title":"2.X.X to 3.X.X - Upgrading","doc":"","ref":"upgrade.html#2-x-x-to-3-x-x"},{"type":"extras","title":"MapSet - Upgrading","doc":"Prior to version **2.2.0**, **ex_aws_dynamo** was encoding **MapSet** data into one of the three Dynamo set types (number, string, and binary), but was decoding those types into Elixir **List** data. In version **2.2.0**, a PR added a boolean configuration option, `:decode_sets`, that would allow the user to specify that these types should be decoded into **MapSet** as well.\n\nIn this major release, Dynamo set types decode to Elixir **MapSet** by default, so the `:decode_sets` configuration option can be removed if it was included. If it this option was not included or was explicitly set to `false`, you'll need to be prepared to handle any decoded Dynamo sets as **MapSet** instead of **List**.","ref":"upgrade.html#mapset"},{"type":"extras","title":"Jason - Upgrading","doc":"Several months ago, `ex_aws` switched its default JSON codec from **Poison** to **Jason** (see https://github.com/ex-aws/ex_aws/pull/658). In this major release, the encoder used for `dev` and `test` environments has been changed to **Jason**, and the README file has been updated to show **Jason** in use instead of **Poison**, and refers the user to the `ex_aws` docs for information about how to provide a custom JSON codec.","ref":"upgrade.html#jason"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}